
//-------------------------------------------------------------------------------------
/*
    GS SHADOWMAP GENERATOR for GS 6.4 +

 An adaptation of code found on the GS wiki by Joseph Duffey 2005 (Josiah).

 1. The model must have a color skin as ent skin 1 and a fully white (255,255,255)
    24 bit tga as its second ent skin.

 2. The model must be square and the vertex numbers of all four corners known. 
    These need to be set in the script below.

 3. The model is imported into wed replacing the current model and the "terrain" 
    action is assigned to it. Then the project must be rebuilt.

 4. The suns offset must be set in the script below, it can be dynamic or static. 

     Dynamic; The suns rays are cast at a constant relative angle to the terrain
              as if the sun is very far away. The sun will move to keep its 
              offset relationship to each pixel.

     Static;  The suns rays are cast from one vector in the sky to the terrain             
              as if the sun is very close. The sun will stay put, and cast rays
              from that position only.
   
    Dynamic provides much more realistic results IMO, but experiment.

 5. The shadows brightness can be set, and the amount of "blurring" done also. 

 6. The created shadow map is the same size as the second ent skin and created in 
    the "output" sub folder.

 7. Use mouse and it's 3 buttons to fly a camera around. Control the pan and tilt
    with mouse movement. Use left click to go forward, right click to go backwards
    and middle to stop. Arrow keys to strafe or raise / descend.


Notes:

 The uv mapping alignment makes a difference! If the shadows are not drawn right,
 and the vertex numbers are correctly entered, it may be that the mapping needs
 rotated 90 or 180 degrees. For mapping in max, the uv gizmo must be rotated 90 CW
 so the tab is pointing to the right. (vertex 1 is usually in the lower left corner), 

 You can get the corner vertex ID numbers by selecting each of them in MED. 

*/
//-------------------------------------------------------------------------------------
//
// Settings
//
//-------------------------------------------------------------------------------------
//Set models corner vert numbers here (you can find them in MED)
var TL_vt= 1; //Top left vert#
var TR_vt= 32; //Top right vert#
var BL_vt= 993; //Bottom left vert#
var BR_vt= 1024; //Bottom right vert#

//Set shadow color here, higher values = lighter shadows 
var shadow_brightness=164;

//Set shadow blur passes here, (default 2), 0 = no blur 
var B_pass = 2;

//Set the tga bit depth (24 or 32) Some shaders need the 32 bit which has an alpha channel
var Bit_depth = 24;

//Set dynamic sun on (1), or static sun(0), here
var dynamic_sun = 1;

//Set dynamic sun XYZ offset here. Z must be higher than terrain apex
var Dyn_sun[3] = 1500,1500,1500;

//Set static sun offset FROM MODELS ORIGIN here. Z must be higher than terrain apex
var Stat_sun[3] = 2500,2500,2500;

//-------------------------------------------------------------------------------------
// 
//  End Settings
//
//-------------------------------------------------------------------------------------

Path "Images";
Path "Models";
Path "Script";

function display_stat_msg(str);
function write8(byte);
function write16(short);
function write_tga();
function Set_Suns_Angle();
function blur(passes);
function getxyz(px,py);
function generate_shadows();

include <message.wdl>;
include <output_tga.wdl>;
include <shadow_gen.wdl>;

//====================================================================================
// CAMERA
//====================================================================================

action fly_cam
{
  var fly_speed = 0;
  my.tilt = -45; //initial tilt  
  my.invisible = on;

  while(1)
   {
     if(mouse_left){fly_speed += 0.1*time;}; // speed up
     if(mouse_right){fly_speed -= 0.1*time;}; // slow down
     if(mouse_middle){fly_speed = 0;};
     my.pan -= mouse_force.x*time*35;
     my.tilt += mouse_force.y*time*30;
     temp.x = fly_speed; 
     temp.y = key_cul-key_cur;
     temp.z = key_cuu-key_cud; 
     move_friction = 0; // no friction for gliding along surfaces
     c_move(my, temp.x, nullvector, ignore_passable + glide);
     vec_set(camera.x, my.x);
     vec_set(camera.pan, my.pan);
     wait(1);
   }
}

//====================================================================================
// MAIN
//====================================================================================

function main()
{
  //INITIALIZE WORLD
  video_set(sys_metrics(0),sys_metrics(1),32,1); //set to desktop resolution
  camera.aspect = (screen_size.x/screen_size.y) / 1.333;//auto pixel aspect ratio
  wait(3);//if triple buffer used
  level_load("main.wmb");
  wait(1);
  ent_create("sun_icon.tga",Stat_sun.x, Sun_icon);//create the sun
  ent_create("cube.mdl",vector(-2600,0,2600), fly_cam);//create the camera

  //MAIN SHADOW GEN ROUTINE
  wait(5);
  display_stat_msg("Press Enter To Begin.");
  while(key_enter == 0){wait(1);}

  display_stat_msg("Generating shadows...");
  generate_shadows();
  while(continue_flag == 0){wait(1);}

  Set_Suns_Angle();//set sun to initial offset

  display_stat_msg("Blurring Shadows...");
  blur(B_pass); 
  while(continue_flag == 0){wait(1);}

  display_stat_msg("Writing output TGA...");
  write_tga();
  while(continue_flag == 0){wait(1);}

  sleep(1);
  beep();
  display_stat_msg("Done!");
  sleep(1);
  display_stat_msg("");
}

